@@ -37,6 +37,7 @@
 #endif
 
 #include <stdio.h>
+#include <cstring>
 
 #ifndef curl_free
 #define curl_free free
@@ -50,7 +51,7 @@ namespace MSN
     static size_t msn_handle_curl_header(void *ptr, size_t size, size_t nmemb, void *stream) ;    
             
     NotificationServerConnection::NotificationServerConnection(NotificationServerConnection::AuthData & auth_, Callbacks & cb_)
-        : Connection(), auth(auth_), externalCallbacks(cb_), _connectionState(NS_DISCONNECTED)
+        : Connection(), auth(auth_), externalCallbacks(cb_), _connectionState(NS_DISCONNECTED), _nextPing(50)
     {
         registerCommandHandlers();
     }
@@ -152,6 +153,7 @@ namespace MSN
             commandHandlers["CHL"] = &NotificationServerConnection::handle_CHL;
             commandHandlers["ILN"] = &NotificationServerConnection::handle_ILN;
             commandHandlers["NLN"] = &NotificationServerConnection::handle_NLN;
+            commandHandlers["NOT"] = &NotificationServerConnection::handle_NOT;
             commandHandlers["FLN"] = &NotificationServerConnection::handle_FLN;
             commandHandlers["MSG"] = &NotificationServerConnection::handle_MSG;
             commandHandlers["ADG"] = &NotificationServerConnection::handle_ADG;
@@ -282,6 +284,13 @@ namespace MSN
         this->myNotificationServer()->externalCallbacks.buddyChangedStatus(this, args[2], decodeURL(args[3]), buddyStatusFromString(args[1]));
     }
     
+    void NotificationServerConnection::handle_NOT(std::vector<std::string> & args)
+    {
+        this->assertConnectionStateIsAtLeast(NS_CONNECTED);
+        int notlen = decimalFromString(args[1]);
+        this->readBuffer = this->readBuffer.substr(notlen);
+    }
+    
     void NotificationServerConnection::handle_FLN(std::vector<std::string> & args)
     {
         this->assertConnectionStateIsAtLeast(NS_CONNECTED);
@@ -515,7 +524,7 @@ public:
             this->socketConnectionCompleted();
         
         std::ostringstream buf_;
-        buf_ << "VER " << this->trID << " MSNP8\r\n";
+        buf_ << "VER " << this->trID << " MSNP9\r\n";
         if (this->write(buf_) != buf_.str().size())
             return;
         
@@ -570,10 +579,12 @@ public:
                 else
                     dataLength = decimalFromString(args[1]);
 
-                if (this->readBuffer.find("\r\n") + 2 + dataLength > this->readBuffer.size())
+                if (this->readBuffer.find("\r\n", 0) + 2 + dataLength > this->readBuffer.size()) {
+                    this->myNotificationServer()->externalCallbacks.showError(this, "Could not read buffer: too small");
                     return;
+                }
             }
-            this->readBuffer = this->readBuffer.substr(this->readBuffer.find("\r\n") + 2);
+            this->readBuffer = this->readBuffer.substr(this->readBuffer.find("\r\n", 0) + 2);
             int trid = 0;
             
             if (args.size() >= 6 && args[0] == "XFR" && args[2] == "NS")
@@ -597,12 +608,12 @@ public:
                 return;
             }
             
-            if (args.size() >= 1 && args[0] == "QNG")
+            if (args.size() >= 2 && args[0] == "QNG")
             {
                 // QNG
                 //  0
                 
-                // ping response, ignore
+                this->_nextPing = decimalFromString(args[1]);
                 return;
             }
             
@@ -770,7 +781,7 @@ public:
         connectinfo * info = (connectinfo *) data;
         this->removeCallback(trid);
         
-        if (args.size() >= 3 && args[0] != "VER" || args[2] != "MSNP8") // if either *differs*...
+        if (args.size() >= 3 && args[0] != "VER" || args[2] != "MSNP9") // if either *differs*...
         {
             this->myNotificationServer()->externalCallbacks.showError(NULL, "Protocol negotiation failed");
             delete info;
@@ -779,7 +790,7 @@ public:
         }
         
         std::ostringstream buf_;
-        buf_ << "CVR " << this->trID << " 0x0409 winnt 5.2 i386 MSNMSGR 6.0.0250 MSMSGS " << info->username << "\r\n";
+        buf_ << "CVR " << this->trID << " 0x0409 winnt 5.2 i386 MSNMSGR 7.5.0324 MSMSGS " << info->username << "\r\n";
         if (this->write(buf_) != buf_.str().size())
             return;
         this->addCallback(&NotificationServerConnection::callback_RequestUSR, this->trID++, (void *) data);
