@@ -31,6 +31,7 @@
 #include "icqface.h"
 #include "imlogger.h"
 #include "icqgroups.h"
+#include "icqcontacts.h"
 
 #include "accountmanager.h"
 #include "eventmanager.h"
@@ -54,7 +55,8 @@ static const Status stat2int[imstatus_size] = {
     STATUS_DND,
     STATUS_OCCUPIED,
     STATUS_NA,
-    STATUS_AWAY
+    STATUS_AWAY,
+    STATUS_NA
 };
 
 icqhook::icqhook(): abstracthook(icq) {
@@ -1109,11 +1111,20 @@ void icqhook::messaged_cb(MessageEvent *ev) {
     if(ev->getType() == MessageEvent::Normal) {
 	NormalMessageEvent *r = static_cast<NormalMessageEvent *>(ev);
 
-	text = r->getMessage();
-	if(text.substr(0, 6) == "{\\rtf1")
-	    text = fixicqrtf(striprtf(text, conf.getconvertfrom(proto)));
+	bool converted = false;
 
-	em.store(immessage(ic, imevent::incoming, rusconv("wk", text), r->getTime()));
+	// check for UCS-2BE encoding in message
+	if (r->getEncoding() == (unsigned int)0x0002) {
+	    string msg = r->getMessage();
+	    text = rusconv("ck", msg);
+	    converted = true;
+	}
+	else
+	    text = r->getMessage();
+	if(text.substr(0, 6) == "{\\rtf1")
+	    text = fixicqrtf(striprtf(text, converted?conf.getconvertto(proto):conf.getconvertfrom(proto)));
+
+	em.store(immessage(ic, imevent::incoming, converted?text:rusconv("wk", text), r->getTime()));
 
     } else if(ev->getType() == MessageEvent::URL) {
 	URLMessageEvent *r = static_cast<URLMessageEvent *>(ev);
@@ -1290,9 +1301,10 @@ void icqhook::contact_userinfo_change_signal_cb(UserInfoChangeEvent *ev) {
 
     } else {
 	char *p;
-	string lastip, sbuf;
+	string lastip, sbuf, oldip;
 	struct in_addr addr;
 
+	oldip = c->getlastip();
 	addr.s_addr = ntohl(ic->getExtIP());
 	if(p = inet_ntoa(addr)) lastip = p;
 
@@ -1309,6 +1321,15 @@ void icqhook::contact_userinfo_change_signal_cb(UserInfoChangeEvent *ev) {
 
 	    c->setlastip(lastip);
 	}
+	else
+        {
+            if ((c->getstatus() != offline) && (oldip.find_first_of("*")))
+            {
+		oldip.insert(0, "* ");
+                c->setlastip(oldip);
+            }
+        }
+
     }
 }
 
