@@ -26,6 +26,7 @@
 
 #ifdef BUILD_IRC
 
+#include "icqgroups.h"
 #include "icqface.h"
 #include "icqcontacts.h"
 #include "imlogger.h"
@@ -33,14 +34,19 @@
 #include "accountmanager.h"
 #include "eventmanager.h"
 
+#include "centerim.h"
+
 #include <iterator>
+#include <sstream>
+
+#define NOTIFBUF 512
 
 // ----------------------------------------------------------------------------
 
 irchook irhook;
 
 irchook::irchook()
-    : abstracthook(irc), handle(firetalk_create_handle(FP_IRC, 0)),
+    : abstracthook(irc), handle(firetalk_create_handle(firetalk_find_protocol("IRC"), 0)),
       fonline(false), flogged(false), ourstatus(offline)
 {
     fcapabs.insert(hookcapab::setaway);
@@ -157,7 +163,7 @@ void irchook::main() {
 void irchook::getsockets(fd_set &rfds, fd_set &wfds, fd_set &efds, int &hsocket) const {
     int *r, *w, *e, *sr, *sw, *se;
 
-    firetalk_getsockets(FP_IRC, &sr, &sw, &se);
+    firetalk_getsockets(firetalk_find_protocol("IRC"), &sr, &sw, &se);
 
     for(r = sr; *r; r++) {
 	if(*r > hsocket) hsocket = *r;
@@ -184,7 +190,7 @@ bool irchook::isoursocket(fd_set &rfds, fd_set &wfds, fd_set &efds) const {
     int *r, *w, *e, *sr, *sw, *se;
 
     if(online()) {
-	firetalk_getsockets(FP_IRC, &sr, &sw, &se);
+	firetalk_getsockets(firetalk_find_protocol("IRC"), &sr, &sw, &se);
 
 	for(r = sr; *r; r++) res = res || FD_ISSET(*r, &rfds);
 	for(w = sw; *w; w++) res = res || FD_ISSET(*w, &wfds);
@@ -296,7 +302,10 @@ void irchook::sendnewuser(const imcontact &ic) {
 
     if(online()) {
 	if(!ischannel(ic)) {
-	    firetalk_im_add_buddy(handle, ic.nickname.c_str());
+	    vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), clist.get(ic)->getgroupid()); 
+	    if(ig != groups.end()) {
+	    	firetalk_im_add_buddy(handle, ic.nickname.c_str(), ig->getname().c_str(), "");
+	    }
 	    requestinfo(ic);
 	} else {
 	    vector<channelInfo> ch = getautochannels();
@@ -338,12 +347,13 @@ void irchook::setautostatus(imstatus st) {
 		switch(st) {
 		    case away:
 		    case notavail:
+		    case outforlunch:
 		    case occupied:
-			firetalk_set_away(handle, conf.getawaymsg(irc).c_str());
+			firetalk_set_away(handle, conf.getawaymsg(irc).c_str(), 0);
 			break;
 
 		    default:
-			firetalk_set_away(handle, 0);
+			firetalk_set_away(handle, 0, 0);
 			break;
 		}
 	    }
@@ -400,25 +410,25 @@ void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	while(!(room = getword(rooms)).empty()) {
 	    if(room[0] != '#') room.insert(0, "#");
 	    searchchannels.push_back(room);
-	    ic = find(channels.begin(), channels.end(), room);
+            ic = find(irhook.channels.begin(), irhook.channels.end(), room);
 
-	    if(ic == channels.end()) {
-		channels.push_back(channelInfo(room));
-		ic = channels.end()-1;
+	    if(ic == irhook.channels.end()) {
+		irhook.channels.push_back(channelInfo(room));
+		ic = irhook.channels.end()-1;
 	    }
 
 	    if(!ic->joined) {
-		firetalk_chat_join(handle, room.c_str(), ic->passwd.c_str());
+		firetalk_chat_join(handle, room.c_str());
 		ready = false;
 	    }
 
 	    if(emailsub.empty() && namesub.empty()) {
-		if(ic->fetched) {
+	/*	if(ic->fetched) {*/
 		    ic->nicks.clear();
 		    firetalk_chat_listmembers(handle, room.c_str());
-		} else {
+/*		} else {
 		    ready = false;
-		}
+		}*/
 
 	    } else {
 		ready = false;
@@ -579,7 +589,7 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
 	if(iac->joined) {
 	    r = ic == channels.end();
 	    if(!r) r = !ic->joined;
-	    if(r) firetalk_chat_join(irhook.handle, iac->name.c_str(), iac->passwd.c_str());
+	    if(r) firetalk_chat_join(irhook.handle, iac->name.c_str());
 	}
     }
 
@@ -594,7 +604,7 @@ void irchook::requestawaymsg(const imcontact &ic) {
 void irchook::rawcommand(const string &cmd) {
     int *r, *w, *e, sock;
     if(online()) {
-	firetalk_getsockets(FP_IRC, &r, &w, &e);
+	firetalk_getsockets(firetalk_find_protocol("IRC"), &r, &w, &e);
 
 	if(*r) {
 	    write(*r, cmd.c_str(), cmd.size());
@@ -609,11 +619,11 @@ void irchook::rawcommand(const string &cmd) {
 
 void irchook::channelfatal(string room, const char *fmt, ...) {
     va_list ap;
-    char buf[1024];
+    char buf[NOTIFBUF];
     vector<channelInfo>::iterator i;
 
     va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
+    vsnprintf(buf, NOTIFBUF, fmt, ap);
     va_end(ap);
 
     if(room.substr(0, 1) != "#")
@@ -725,13 +735,16 @@ void irchook::connected(void *conn, void *cli, ...) {
 
 	if(c->getdesc().pname == irc)
 	if(!ischannel(c)) {
-	    firetalk_im_add_buddy(irhook.handle, c->getdesc().nickname.c_str());
+	    vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), clist.get(c)->getgroupid()); 
+	    if(ig != groups.end()) {
+	        firetalk_im_add_buddy(irhook.handle, c->getdesc().nickname.c_str() , ig->getname().c_str(), "");
+	    }
 	}
     }
 
     for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic) {
 	if(ic->joined) {
-	    firetalk_chat_join(irhook.handle, ic->name.c_str(), ic->passwd.c_str());
+	    firetalk_chat_join(irhook.handle, ic->name.c_str());
 	}
     }
 
@@ -1196,7 +1209,7 @@ void irchook::errorhandler(void *connection, void *cli, ...) {
 void irchook::nickchanged(void *connection, void *cli, ...) {
     va_list ap;
     icqcontact *c;
-    char buf[100];
+    char buf[NOTIFBUF];
 
     va_start(ap, cli);
     char *oldnick = va_arg(ap, char *);
@@ -1218,7 +1231,7 @@ void irchook::nickchanged(void *connection, void *cli, ...) {
 
 	    }
 
-	    sprintf(buf, _("The user has changed their nick from %s to %s"), oldnick, newnick);
+	    snprintf(buf, NOTIFBUF, _("The user has changed their nick from %s to %s"), oldnick, newnick);
 	    em.store(imnotification(c, buf));
 	}
     }
@@ -1247,25 +1260,26 @@ void irchook::subrequest(void *conn, void *cli, const char * const nick,
 const char * const command, const char * const args) {
 
     if(!strcmp(command, "VERSION")) {
-	firetalk_subcode_send_reply(conn, nick, "VERSION", PACKAGE " " VERSION);
-
+	ostringstream args_stream;
+	args_stream << PACKAGE << " " << centerim::version;
+	firetalk_subcode_send_reply(conn, nick, "VERSION", args_stream.str().c_str());
     }
 }
 
 void irchook::subreply(void *conn, void *cli, const char * const nick,
 const char * const command, const char * const args) {
-    char buf[512];
+    char buf[NOTIFBUF];
 
     if(!strcmp(command, "PING")) {
 	map<string, time_t>::iterator i = irhook.pingtime.find(up(nick));
 
 	if(i != irhook.pingtime.end()) {
-	    sprintf(buf, _("PING reply from the user: %d second(s)"), time(0)-i->second);
+	    snprintf(buf, NOTIFBUF, _("PING reply from the user: %d second(s)"), time(0)-i->second);
 	    em.store(imnotification(imcontact(nick, irc), buf));
 	}
 
     } else if(!strcmp(command, "VERSION")) {
-	sprintf(buf, _("The remote is using %s"), args);
+	snprintf(buf, NOTIFBUF, _("The remote is using %s"), args);
 	em.store(imnotification(imcontact(nick, irc), buf));
 
     }
@@ -1378,8 +1392,8 @@ void irchook::chatuserjoined(void *conn, void *cli, ...) {
 	if(strlen(email))
 	    uname += (string) " (" + email + ")";
 
-	char buf[512];
-	sprintf(buf, _("%s has joined."), uname.c_str());
+	char buf[NOTIFBUF];
+	snprintf(buf, NOTIFBUF, _("%s has joined."), uname.c_str());
 	em.store(imnotification(imcontact(room, irc), buf));
     }
 }
@@ -1395,14 +1409,16 @@ void irchook::chatuserleft(void *conn, void *cli, ...) {
 
     if(conf.getourid(irc).nickname != who) {
 	string text;
-	char buf[512];
+	string text2;
+	char buf[NOTIFBUF];
 
-	sprintf(buf, _("%s has left"), who); text = buf;
+	snprintf(buf, NOTIFBUF, _("%s has left"), who); text = buf;
 
 	if(reason)
 	if(strlen(reason)) {
 	    if(strlen(reason) > 450) reason[450] = 0;
-	    sprintf(buf, _("reason: %s"), reason);
+    	    text2 = irhook.rushtmlconv( "wk", reason );
+	    snprintf(buf, NOTIFBUF, _("reason: %s"), reason);
 	    text += (string) "; " + buf + ".";
 	}
 
@@ -1422,13 +1438,13 @@ void irchook::chatuserkicked(void *conn, void *cli, ...) {
 
     if(conf.getourid(irc).nickname != who) {
 	string text;
-	char buf[512];
+	char buf[NOTIFBUF];
 
-	sprintf(buf, _("%s has been kicked by %s"), who, by); text = buf;
+	snprintf(buf, NOTIFBUF, _("%s has been kicked by %s"), who, by); text = buf;
 
 	if(reason)
 	if(strlen(reason)) {
-	    sprintf(buf, _("reason: %s"), reason);
+	    snprintf(buf, NOTIFBUF, _("reason: %s"), reason);
 	    text += (string) "; " + buf + ".";
 	}
 
@@ -1451,13 +1467,14 @@ void irchook::chatgottopic(void *conn, void *cli, ...) {
 	return;
 
     string text;
-    char buf[1024];
-    sprintf(buf, _("Channel topic now is: %s"), topic);
+    char buf[NOTIFBUF];
+    text = irhook.rushtmlconv( "wk", topic );
+    snprintf(buf, NOTIFBUF, _("Channel topic now is: %s"), text.c_str());
     text = buf;
 
     if(author)
     if(strlen(author)) {
-	sprintf(buf, _("set by %s"), author);
+	snprintf(buf, NOTIFBUF, _("set by %s"), author);
 	text += (string) "; " + buf + ".";
     }
 
@@ -1474,8 +1491,8 @@ void irchook::chatuseropped(void *conn, void *cli, ...) {
     va_end(ap);
 
     if(by) {
-	char buf[512];
-	sprintf(buf, _("%s has been opped by %s."), who, by);
+	char buf[NOTIFBUF];
+	snprintf(buf, NOTIFBUF, _("%s has been opped by %s."), who, by);
 	em.store(imnotification(imcontact(room, irc), buf));
     }
 }
@@ -1490,8 +1507,8 @@ void irchook::chatuserdeopped(void *conn, void *cli, ...) {
     va_end(ap);
 
     if(by) {
-	char buf[512];
-	sprintf(buf, _("%s has been deopped by %s."), who, by);
+	char buf[NOTIFBUF];
+	snprintf(buf, NOTIFBUF, _("%s has been deopped by %s."), who, by);
 	em.store(imnotification(imcontact(room, irc), buf));
     }
 }
@@ -1504,10 +1521,9 @@ void irchook::chatopped(void *conn, void *cli, ...) {
     char *by = va_arg(ap, char *);
     va_end(ap);
 
-    char buf[512];
-    if(by) sprintf(buf, _("%s has opped us."), by);
-	else strcpy(buf, _("you are an op here"));
-
+    char buf[NOTIFBUF];
+    if(by) snprintf(buf, NOTIFBUF, _("%s has opped us."), by);
+	else snprintf(buf, NOTIFBUF, _("you are an op here"));
     em.store(imnotification(imcontact(room, irc), buf));
 }
 
@@ -1519,8 +1535,8 @@ void irchook::chatdeopped(void *conn, void *cli, ...) {
     char *by = va_arg(ap, char *);
     va_end(ap);
 
-    char buf[512];
-    sprintf(buf, _("%s has deopped us."), by);
+    char buf[NOTIFBUF];
+    snprintf(buf, NOTIFBUF, _("%s has deopped us."), by);
     em.store(imnotification(imcontact(room, irc), buf));
 }
 
