@@ -32,10 +32,18 @@
 #include "eventmanager.h"
 #include "icqgroups.h"
 #include "impgp.h"
+#include "icqcontacts.h"
+
+#ifdef HAVE_LIBOTR
+  #include "imotr.h"
+#endif 
+
 
 #define DEFAULT_CONFSERV "conference.jabber.org"
 #define PERIOD_KEEPALIVE 30
 
+#define NOTIFBUF 512
+
 static void jidsplit(const string &jid, string &user, string &host, string &rest) {
     int pos;
     user = jid;
@@ -49,14 +57,20 @@ static void jidsplit(const string &jid, string &user, string &host, string &rest
 	host = user.substr(pos+1);
 	user.erase(pos);
     }
+    else {
+        host = user;
+        user = (string)"";
+    }
 }
 
 static string jidtodisp(const string &jid) {
     string user, host, rest;
     jidsplit(jid, user, host, rest);
 
-    if(!host.empty())
+    if(!user.empty())
 	user += (string) "@" + host;
+    else
+	user = host;
 
     return user;
 }
@@ -75,7 +89,10 @@ string jabberhook::jidnormalize(const string &jid) const {
     if(host.empty())
 	host = conf.getourid(proto).server;
 
-    user += (string) "@" + host;
+    if (user.empty())
+	user = host;
+    else
+	user += (string) "@" + host;
     if(!rest.empty()) user += (string) "/" + rest;
     return user;
 }
@@ -102,6 +119,9 @@ jabberhook::~jabberhook() {
 }
 
 void jabberhook::init() {
+    char rnd[9];
+    snprintf(rnd, 9, "%X%X", rand()%0xFFFF, rand()%0xFFFF);
+    uuid = rnd;
     manualstatus = conf.getstatus(proto);
 }
 
@@ -292,6 +312,13 @@ bool jabberhook::send(const imevent &ev) {
 	}
 #endif
 
+#ifdef HAVE_LIBOTR
+    if (!otr.send_message(proto, jidnormalize(ev.getcontact().nickname), text))
+    {
+        return true;
+    }
+#endif
+
 	/* TODO: do these really needs to be copied? */
 	char *cjid = strdup(jidnormalize(c->getdesc().nickname).c_str());
 	char *ctext = strdup(text.c_str());
@@ -362,7 +389,7 @@ void jabberhook::sendnewuser(const imcontact &ic, bool report) {
 	    vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), c->getgroupid());
 	    if(ig != groups.end()) {
 		z = xmlnode_insert_tag(z, "group");
-		xmlnode_insert_cdata(z, ig->getname().c_str(), (unsigned) -1);
+		xmlnode_insert_cdata(z, rusconv("ku", ig->getname()).c_str(), (unsigned) -1);
 		roster[cjid] = ig->getname();
 	    }
 	}
@@ -380,7 +407,10 @@ void jabberhook::sendnewuser(const imcontact &ic, bool report) {
 
 	    string u, h, r;
 	    jidsplit(icc.nickname, u, h, r);
-	    c->setdispnick(u);
+	    if (!u.empty())
+	    	c->setdispnick(u);
+	    else
+		c->setdispnick(icc.nickname);
 	}
 
 	requestinfo(c);
@@ -485,6 +515,7 @@ void jabberhook::setautostatus(imstatus st) {
 		case away:
 		case dontdisturb:
 		case occupied:
+		case outforlunch:
 		case notavail:
 		    msg = conf.getawaymsg(proto);
 	    }
@@ -618,6 +649,7 @@ void jabberhook::setjabberstatus(imstatus st, string msg) {
 	    xmlnode_insert_cdata(xmlnode_insert_tag(x, "show"), "chat", (unsigned) -1);
 	    break;
 
+	case outforlunch:
 	case notavail:
 	    xmlnode_insert_cdata(xmlnode_insert_tag(x, "show"), "xa", (unsigned) -1);
 	    break;
@@ -640,6 +672,14 @@ void jabberhook::setjabberstatus(imstatus st, string msg) {
 
     if(msg.empty())
 	msg = imstatus2str(st);
+	int pos = 0;
+	while( (pos = msg.find( '\r' )) != string::npos ) {
+		if( msg[pos+1] == '\n' ) {
+			msg.erase(pos, 1);
+		} else {
+			msg.replace(pos, 1, "\n");
+		}
+	}
 
     xmlnode_insert_cdata(xmlnode_insert_tag(x, "status"),
 	rusconv("ku", msg).c_str(), (unsigned) -1);
@@ -711,7 +751,8 @@ vector<string> jabberhook::getservices(servicetype::enumeration st) const {
     }
 
     while(ia != agents.end()) {
-	if(ia->params[pt].enabled) r.push_back(ia->name);
+	if(ia->params[pt].enabled)
+	    r.push_back(ia->name);
 	++ia;
     }
 
@@ -743,12 +784,67 @@ vector<pair<string, string> > jabberhook::getregparameters(const string &agentna
 void jabberhook::gotagentinfo(xmlnode x) {
     xmlnode y;
     string name, data, ns;
-    agent::param_type pt;
+    //agent::param_type pt;
     vector<agent>::iterator ia = jhook.agents.begin();
-    const char *from = xmlnode_get_attrib(x, "from"), *p;
+    const char *from = xmlnode_get_attrib(x, "from"), *p, *q;
 
     if(!from) return;
 
+    while(ia != jhook.agents.end()) {
+	if(ia->jid == from)
+	if(y = xmlnode_get_tag(x, "query")) {
+	    p = xmlnode_get_attrib(y, "xmlns"); if(p) ns = p;
+	    if (ns == NS_DISCOINFO) {
+	    for(y = xmlnode_get_firstchild(y); y; y = xmlnode_get_nextsibling(y)) {
+		p = xmlnode_get_name(y); name = p ? p : "";
+		if (name == "identity") {
+		    if (q = xmlnode_get_attrib(y, "name"))
+			ia->desc = q;
+		    if (q = (xmlnode_get_attrib(y, "category"))) {
+			data = q;
+			if (data == "conference") {
+			    ia->type = agent::atGroupchat;
+			} else if (data == "gateway") {
+			    ia->type = agent::atTransport;
+			}
+		    }
+		} else if (name == "feature") {
+		    if ((q = xmlnode_get_attrib(y, "var")) && (!strcmp(q, NS_SEARCH))) { // can do search, ask for parameters
+			ia->params[agent::ptSearch].enabled = true;
+	    		ia->params[agent::ptSearch].paramnames.clear();
+			xmlnode z = jutil_iqnew(JPACKET__GET, NS_SEARCH);
+			xmlnode_put_attrib(z, "to", from);
+			jab_send(jc, z);
+			xmlnode_free(z);
+		    }
+		}
+	    }
+	    break;
+	    } else if (ns == NS_SEARCH) { // probably agent info with jabber:iq:search
+		for(y = xmlnode_get_firstchild(y); y; y = xmlnode_get_nextsibling(y)) {
+		    p = xmlnode_get_name(y); name = p ? p : "";
+		    p = xmlnode_get_data(y); data = p ? p : "";
+		    if (name == "item") // it's probably not answer with parameters
+			break;
+		    if (name == "instructions") {
+			ia->params[agent::ptSearch].instruction = data;
+		    } else if ((name == "x") && (NSCHECK(y, "jabber:x:data"))) {
+			continue;
+		    } else if (name == "key") {
+		    	ia->params[agent::ptSearch].key = data;
+		    } else {
+			ia->params[agent::ptSearch].paramnames.push_back(make_pair(name, data));
+		    }
+	    	}
+		if ((name != "item") && ia->params[agent::ptSearch].paramnames.empty())
+		    agents.erase(ia);
+		break;
+	    }
+	}
+	++ia;
+    }
+
+/*
     while(ia != jhook.agents.end()) {
 	if(ia->jid == from)
 	if(y = xmlnode_get_tag(x, "query")) {
@@ -777,7 +873,7 @@ void jabberhook::gotagentinfo(xmlnode x) {
 	}
 	++ia;
     }
-
+*/
 }
 
 void jabberhook::lookup(const imsearchparams &params, verticalmenu &dest) {
@@ -916,7 +1012,8 @@ void jabberhook::gotsearchresults(xmlnode x) {
 }
 
 void jabberhook::gotloggedin() {
-    xmlnode x;
+    xmlnode x, y;
+    char *cid;
 
     flogged = true;
 
@@ -925,6 +1022,26 @@ void jabberhook::gotloggedin() {
 //  jab_send(jc, x);
 //  xmlnode_free(x);
 
+    char *server = strdup(jc->user->server);
+
+    jhook.agents.push_back(agent(server, server, "", agent::atUnknown));
+
+    x = jutil_iqnew(JPACKET__GET, NS_DISCOINFO);
+    cid = jab_getid(jc);
+    xmlnode_put_attrib(x, "id", cid);
+    xmlnode_put_attrib(x, "to", server);
+    jab_send(jc, x);
+    xmlnode_free(x);
+
+    x = jutil_iqnew(JPACKET__GET, NS_DISCOITEMS);
+    cid = jab_getid(jc);
+    xmlnode_put_attrib(x, "id", cid);
+    xmlnode_put_attrib(x, "to", server);
+    jab_send(jc, x);
+    xmlnode_free(x);
+    free(server);
+
+
     x = jutil_iqnew(JPACKET__GET, NS_ROSTER);
     xmlnode_put_attrib(x, "id", "Roster");
     jab_send(jc, x);
@@ -935,6 +1052,7 @@ void jabberhook::gotroster(xmlnode x) {
     xmlnode y, z;
     imcontact ic;
     icqcontact *c;
+    string grp;
 
     for(y = xmlnode_get_tag(x, "item"); y; y = xmlnode_get_nextsibling(y)) {
 	const char *alias = xmlnode_get_attrib(y, "jid");
@@ -944,20 +1062,24 @@ void jabberhook::gotroster(xmlnode x) {
 
 	z = xmlnode_get_tag(y, "group");
 	if(z) group = xmlnode_get_data(z);
+	grp = group ? rusconv("uk", group) : "";
 
 	if(alias) {
 	    ic = imcontact(jidtodisp(alias), proto);
-	    clist.updateEntry(ic, group ? group : "");
+	    clist.updateEntry(ic, grp);
 
 	    if(c = clist.get(ic)) {
 		if(name) c->setdispnick(rusconv("uk", name)); else {
 		    string u, h, r;
 		    jidsplit(alias, u, h, r);
-		    c->setdispnick(u);
+		    if (!u.empty())
+			c->setdispnick(u);
+		    else
+			c->setdispnick(jidtodisp(alias));
 		}
 	    }
 
-	    roster[jidnormalize(alias)] = group ? group : "";
+	    roster[jidnormalize(alias)] = grp;
 	}
     }
 
@@ -1076,7 +1198,7 @@ void jabberhook::sendupdateuserinfo(const icqcontact &c) {
 		    mi.gender == genderFemale ? "Female" : "");
 
 		if(mi.birth_year && mi.birth_month && mi.birth_day) {
-		    sprintf(cbuf, "%04d-%02d-%02d", mi.birth_year, mi.birth_month, mi.birth_day);
+		    snprintf(cbuf, sizeof(cbuf), "%04d-%02d-%02d", mi.birth_year, mi.birth_month, mi.birth_day);
 		    vcput(y, "BDAY", cbuf);
 		}
 
@@ -1132,7 +1254,7 @@ void jabberhook::gotmessage(const string &type, const string &from, const string
     string u, h, r, body(abody);
     jidsplit(from, u, h, r);
 
-    imcontact ic(jidtodisp(u + "@" + h), proto), chic;
+    imcontact ic(jidtodisp(from), proto), chic;
 
     if(clist.get(chic = imcontact((string) "#" + ic.nickname, proto))) {
 	ic = chic;
@@ -1152,6 +1274,10 @@ void jabberhook::gotmessage(const string &type, const string &from, const string
 	}
     }
 #endif
+    
+#ifdef HAVE_LIBOTR
+    if (!otr.receive_message(proto, from, body)) return;
+#endif
 
     em.store(immessage(ic, imevent::incoming, rusconv("uk", body)));
 }
@@ -1171,7 +1297,7 @@ void jabberhook::updatecontact(icqcontact *c) {
 	vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), c->getgroupid());
 	if(ig != groups.end()) {
 	    y = xmlnode_insert_tag(y, "group");
-	    xmlnode_insert_cdata(y, ig->getname().c_str(), (unsigned) -1);
+	    xmlnode_insert_cdata(y, rusconv("ku", ig->getname()).c_str(), (unsigned) -1);
 	}
 
 	jab_send(jc, x);
@@ -1322,11 +1448,8 @@ void jabberhook::gotversion(const imcontact &ic, xmlnode x) {
 	    vinfo += rusconv("uk", p);
 	}
 
-	if(vinfo.size() > 128)
-	    vinfo.erase(128);
-
-	char buf[256];
-	sprintf(buf, _("The remote is using %s"), vinfo.c_str());
+	char buf[NOTIFBUF];
+	snprintf(buf, NOTIFBUF, _("The remote is using %s"), vinfo.c_str());
 	em.store(imnotification(ic, buf));
     }
 }
@@ -1354,7 +1477,7 @@ string jabberhook::getourjid() {
 	jid.erase(pos);
 
     if(jid.find("/") == -1)
-	jid += "/centerim";
+	jid += "/centerim" + jhook.uuid;
 
     return jid;
 }
@@ -1536,8 +1659,22 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 			    xmlnode_put_attrib(x, "id", "Agent info");
 			    jab_send(conn, x);
 			    xmlnode_free(x);
+			} else if (ns == NS_SEARCH) {
+			    jhook.gotagentinfo(packet->x);
 			}
-
+		    } else if(ns == NS_DISCOITEMS) {
+			for(y = xmlnode_get_tag(x, "item"); y; y = xmlnode_get_nextsibling(y)) {
+			    if (p = xmlnode_get_attrib(y, "jid")) {
+				jhook.agents.push_back(agent(p, p, _(""), agent::atUnknown));
+				x = jutil_iqnew(JPACKET__GET, NS_DISCOINFO);
+				xmlnode_put_attrib(x, "to", p);
+				jab_send(conn, x);
+				xmlnode_free(x);
+			    }
+			}
+		    }
+		    else if(ns == NS_DISCOINFO) {
+			jhook.gotagentinfo(packet->x);
 		    }
 		}
 
@@ -1552,6 +1689,11 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 		p = xmlnode_get_tag_data(packet->x, "error"); if(p) desc = p;
 
 		switch(code) {
+		    case 501: /* Not Implemented */
+		        if(jhook.regmode) {
+		            jhook.regerr = desc;
+		        }
+		        break;
 		    case 401: /* Unauthorized */
 		    case 302: /* Redirect */
 		    case 400: /* Bad request */
@@ -1564,7 +1706,6 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 		    case 408: /* Request Timeout */
 		    case 409: /* Conflict */
 		    case 500: /* Internal Server Error */
-		    case 501: /* Not Implemented */
 		    case 502: /* Remote Server Error */
 		    case 503: /* Service Unavailable */
 		    case 504: /* Remote Server Timeout */
@@ -1575,7 +1716,7 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 				_("+ [jab] error %d: %s"),
 				code, desc.c_str());
 
-			    if(!jhook.flogged && code != 501) {
+			    if(!jhook.flogged) {
 				close(jhook.jc->fd);
 				jhook.jc->fd = -1;
 			    }
@@ -1590,6 +1731,21 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 	    break;
 
 	case JPACKET_PRESENCE:
+	    if (type == "error")
+	    {
+		if(!jhook.regmode) {
+		    string desc;
+		    int code;
+		    x = xmlnode_get_tag(packet->x, "error");
+		    p = xmlnode_get_attrib(x, "code"); if(p) code = atoi(p);
+		    p = xmlnode_get_tag_data(packet->x, "error"); if(p) desc = p;
+		    face.log(desc.empty() ?
+			_("+ [jab] error %d") :
+			_("+ [jab] error %d: %s"),
+			code, desc.c_str());
+		}
+		break;
+	    }
 	    x = xmlnode_get_tag(packet->x, "show");
 	    ust = available;
 
@@ -1608,7 +1764,7 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 		ust = offline;
 
 	    jidsplit(from, u, h, s);
-	    id = u + "@" + h;
+	    id = jidtodisp(from);
 
 	    if(clist.get(imcontact((string) "#" + id, jhook.proto))) {
 		if(ust == offline) {
@@ -1623,14 +1779,13 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 
 	    } else {
 		icqcontact *c = clist.get(ic);
-
-		if(c)
-		if(c->getstatus() != ust) {
-		    if(c->getstatus() == offline)
+		
+		if(c) {
+		    if(c->getstatus() != ust) {
 			jhook.awaymsgs[ic.nickname] = "";
-
-		    logger.putonline(c, c->getstatus(), ust);
-		    c->setstatus(ust);
+			logger.putonline(c, c->getstatus(), ust);
+			c->setstatus(ust);
+		    }
 
 		    if(x = xmlnode_get_tag(packet->x, "status"))
 		    if(p = xmlnode_get_data(x))
@@ -1643,7 +1798,6 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 		    if(p = xmlnode_get_data(x))
 			c->setpgpkey(pgp.verify(p, jhook.awaymsgs[ic.nickname]));
 #endif
-
 		}
 	    }
 	    break;
