@@ -43,8 +43,6 @@
 #include <netdb.h>
 #include <arpa/inet.h>
 
-#define PERIOD_PING     60
-
 msnhook mhook;
 
 static string nicknormalize(const string &nick) {
@@ -109,9 +107,9 @@ void msnhook::init() {
     convstat[MSN::STATUS_INVISIBLE] = invisible;
     convstat[MSN::STATUS_BUSY] = dontdisturb;
     convstat[MSN::STATUS_ONTHEPHONE] = occupied;
-    convstat[MSN::STATUS_AWAY] = away;
+    convstat[MSN::STATUS_AWAY] = notavail;
     convstat[MSN::STATUS_BERIGHTBACK] = away;
-    convstat[MSN::STATUS_OUTTOLUNCH] = notavail;
+    convstat[MSN::STATUS_OUTTOLUNCH] = outforlunch;
     convstat[MSN::STATUS_IDLE] = away;
 }
 
@@ -145,7 +143,7 @@ void msnhook::disconnect() {
 
 void msnhook::exectimers() {
     if(logged()) {
-	if(timer_current-timer_ping > PERIOD_PING) {
+	if(timer_current-timer_ping > conn.nextPing()) {
 	    try {
 		conn.sendPing();
 		timer_ping = timer_current;
