@@ -47,6 +47,8 @@
 #define PERIOD_REFRESH          60
 #define PERIOD_CLOSE            6
 
+#define NOTIFBUF 512
+
 int yahoohook::yfd::connection_tags = 0;
 
 char pager_host[255], pager_port[255], filetransfer_host[255],
@@ -66,6 +68,7 @@ static int stat2int[imstatus_size] = {
     YAHOO_STATUS_NOTATDESK,
     YAHOO_STATUS_NOTATHOME,
     YAHOO_STATUS_IDLE,
+    YAHOO_STATUS_OUTTOLUNCH,
 };
 
 yahoohook::yahoohook() : abstracthook(yahoo), fonline(false), cid(0) {
@@ -144,7 +147,7 @@ void yahoohook::connect() {
 
     cid = yahoo_init_with_attributes(acc.nickname.c_str(), acc.password.c_str(),
 	"pager_host", acc.server.c_str(),
-	"pager_port", acc.port, 0);
+	"pager_port", acc.port, (char *) NULL);
 
     yahoo_login(cid, stat2int[manualstatus]);
 
@@ -423,10 +426,12 @@ imstatus yahoohook::yahoo2imstatus(int status) {
 	case YAHOO_STATUS_NOTATHOME:
 	case YAHOO_STATUS_NOTINOFFICE:
 	case YAHOO_STATUS_ONVACATION:
-	case YAHOO_STATUS_OUTTOLUNCH:
 	case YAHOO_STATUS_STEPPEDOUT:
 	    st = notavail;
 	    break;
+	case YAHOO_STATUS_OUTTOLUNCH:
+		st = outforlunch;
+		break;
 	case YAHOO_STATUS_INVISIBLE:
 	    st = invisible;
 	    break;
@@ -849,7 +854,7 @@ void yahoohook::got_conf_invite(int id, char *who, char *room, char *msg, YList
     icqconf::imaccount acc = conf.getourid(yahoo);
     string confname = (string) "#" + room, inviter, text;
     vector<string>::iterator ic;
-    char buf[1024];
+    char buf[NOTIFBUF];
     int i;
 
     imcontact cont(confname, yahoo);
@@ -861,7 +866,7 @@ void yahoohook::got_conf_invite(int id, char *who, char *room, char *msg, YList
 	inviter.erase(i);
     }
 
-    sprintf(buf, _("The user %s has invited you to the %s conference, the topic there is: %s"),
+    snprintf(buf, NOTIFBUF, _("The user %s has invited you to the %s conference, the topic there is: %s"),
 	yhook.rusconv("wk", inviter).c_str(),
 	yhook.rusconv("wk", room).c_str(),
 	yhook.rusconv("wk", msg).c_str());
@@ -893,20 +898,20 @@ void yahoohook::got_conf_invite(int id, char *who, char *room, char *msg, YList
 
 void yahoohook::conf_userdecline(int id, char *who, char *room, char *msg) {
     icqcontact *c = clist.get(imcontact((string) "#" + room, yahoo));
-    char buf[512];
+    char buf[NOTIFBUF];
 
     if(c) {
-	sprintf(buf, _("The user %s has declined your invitation to join the conference"), who);
+	snprintf(buf, NOTIFBUF, _("The user %s has declined your invitation to join the conference"), who);
 	em.store(imnotification(c, buf));
     }
 }
 
 void yahoohook::conf_userjoin(int id, char *who, char *room) {
     icqcontact *c = clist.get(imcontact((string) "#" + room, yahoo));
-    char buf[512];
+    char buf[NOTIFBUF];
 
     if(c) {
-	sprintf(buf, _("The user %s has joined the conference"), who);
+	snprintf(buf, NOTIFBUF, _("The user %s has joined the conference"), who);
 
 	if(find(yhook.confmembers[room].begin(), yhook.confmembers[room].end(), who) == yhook.confmembers[room].end())
 	    yhook.confmembers[room].push_back(who);
@@ -917,11 +922,11 @@ void yahoohook::conf_userjoin(int id, char *who, char *room) {
 
 void yahoohook::conf_userleave(int id, char *who, char *room) {
     icqcontact *c = clist.get(imcontact((string) "#" + room, yahoo));
-    char buf[512];
+    char buf[NOTIFBUF];
     vector<string>::iterator im;
 
     if(c) {
-	sprintf(buf, _("The user %s has left the conference"), who);
+	snprintf(buf, NOTIFBUF, _("The user %s has left the conference"), who);
 	em.store(imnotification(c, buf));
 
 	im = find(yhook.confmembers[room].begin(), yhook.confmembers[room].end(), who);
@@ -986,10 +991,10 @@ void yahoohook::game_notify(int id, char *me, char *who, int stat) {
 }
 
 void yahoohook::mail_notify(int id, char *from, char *subj, int cnt) {
-    char buf[1024];
+    char buf[NOTIFBUF];
 
     if(from && subj) {
-	sprintf(buf, _("+ [yahoo] e-mail from %s, %s"), from, subj);
+	snprintf(buf, NOTIFBUF, _("+ [yahoo] e-mail from %s, %s"), from, subj);
 	face.log(buf);
 	clist.get(contactroot)->playsound(imevent::email);
     }
@@ -1143,11 +1148,11 @@ void yahoohook::webcam_data_request(int id, int send) {
 
 int yahoohook::ylog(char *fmt, ...) {
     if(conf.getdebug()) {
-	char buf[512];
+	char buf[NOTIFBUF];
 	va_list ap;
 
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, NOTIFBUF, fmt, ap);
 	va_end(ap);
 
 	face.log(buf);
