@@ -31,6 +31,7 @@
 #include <sys/statvfs.h>
 #endif
 
+#include "centerim.h"
 #include "icqconf.h"
 #include "icqface.h"
 #include "icqcontacts.h"
@@ -50,6 +51,7 @@ icqconf::icqconf() {
     rc = rcdark;
     cm = cmproto;
     fgroupmode = nogroups;
+    fsortmode = sort_by_status_and_activity;
 
     autoaway = autona = 0;
 
@@ -173,8 +175,8 @@ string icqconf::getawaymsg(protocolname pname) const {
     if(r.empty()) {
 	char buf[512];
 
-	sprintf(buf, _("I do really enjoy the default %s away message of %s %s."),
-	    getprotocolname(pname).c_str(), PACKAGE, VERSION);
+	snprintf(buf, sizeof(buf), _("I do really enjoy the default %s away message of %s %s."),
+	    getprotocolname(pname).c_str(), PACKAGE, centerim::version);
 
 	return buf;
     }
@@ -272,11 +274,16 @@ void icqconf::loadkeys() {
 	    of << "bind editor\t\\co\thistory" << endl;
 	    of << "bind editor\t\\cn\tnext_chat" << endl;
 	    of << "bind editor\t\\cb\tprev_chat" << endl;
+	    of << "bind editor\t\\cv\tout_chat" << endl;
 	    of << "bind editor\t\\a?\tinfo" << endl;
 	    of << "bind editor\t<F2>\tshow_urls" << endl;
 	    of << "bind editor\t<F9>\tfullscreen" << endl << endl;
 	    of << "bind editor\t<F4>\tchat_panel_move_up" << endl;
 	    of << "bind editor\t<F5>\tchat_panel_move_down" << endl;
+#ifdef HAVE_LIBOTR
+	    of << "bind editor\t<F7>\totr_start_session" << endl;
+	    of << "bind editor\t<F8>\totr_end_session" << endl;
+#endif        
 	    of << "bind info\t<F2>\tshow_urls" << endl;
 	    of << "bind info\t<F6>\tuser_external_action" << endl;
 
@@ -370,6 +377,7 @@ void icqconf::loadkeys() {
 	    if(param == "history") k.action = key_history; else
 	    if(param == "next_chat") k.action = key_next_chat; else
 	    if(param == "prev_chat") k.action = key_prev_chat; else
+	    if(param == "out_chat") k.action = key_out_chat; else
 	    if(param == "fetch_away_message") k.action = key_fetch_away_message; else
 	    if(param == "user_menu") k.action = key_user_menu; else
 	    if(param == "general_menu") k.action = key_general_menu; else
@@ -399,6 +407,10 @@ void icqconf::loadkeys() {
 	    if(param == "log_panel_move_down")  k.action = key_log_panel_move_down; else
 	    if(param == "chat_panel_move_up")  k.action = key_chat_panel_move_up; else
 	    if(param == "chat_panel_move_down")  k.action = key_chat_panel_move_down; else
+#ifdef HAVE_LIBOTR
+	    if(param == "otr_start_session")  k.action = key_otr_start_session; else
+	    if(param == "otr_end_session")  k.action = key_otr_end_session; else
+#endif        
 		continue;
 
 	    keys.push_back(k);
@@ -439,6 +451,7 @@ void icqconf::loadkeys() {
     if(face.action2key(key_history, section_editor).empty()) 
     if(face.action2key(key_next_chat, section_editor).empty()) 
     if(face.action2key(key_prev_chat, section_editor).empty()) 
+    if(face.action2key(key_out_chat, section_editor).empty()) 
     if(face.action2key(key_info, section_editor).empty()) 
     if(face.action2key(key_show_urls, section_editor).empty()) 
     if(face.action2key(key_fullscreen, section_editor).empty()) 
@@ -482,6 +495,10 @@ void icqconf::loadmainconfig() {
 	    if(param == "group2") fgroupmode = group2; else
 	    if(param == "protocolormode") cm = icqconf::cmproto; else
 	    if(param == "statuscolormode") cm = icqconf::cmstatus; else
+	    if(param == "sort_by_status_and_activity") setsortmode(icqconf::sort_by_status_and_activity); else
+	    if(param == "sort_by_status_and_name") setsortmode(icqconf::sort_by_status_and_name); else
+	    if(param == "sort_by_activity") setsortmode(icqconf::sort_by_activity); else
+	    if(param == "sort_by_name") setsortmode(icqconf::sort_by_name); else
 	    if(param == "smtp") setsmtphost(buf); else
 	    if(param == "browser") setbrowser(browser); else
 	    if(param == "http_proxy") sethttpproxyhost(buf); else
@@ -495,6 +512,7 @@ void icqconf::loadmainconfig() {
 	    if(param == "russian" || param == "convert") initmultiproto(cpconvert, buf, false); else
 	    if(param == "nobidi") setbidi(false); else
 	    if(param == "askaway") askaway = true; else
+	    if(param == "askquit") askquit = true; else
 	    if(param == "logtimestamps") logtimestamps = true; else
 	    if(param == "logonline") logonline = true; else
 	    if(param == "fromcharset") fromcharset = buf; else
@@ -593,6 +611,7 @@ void icqconf::save() {
 	    f << "tocharset\t" << tocharset << endl;
 
 	    if(!getbidi()) f << "nobidi" << endl;
+	    if(getaskquit()) f << "askquit" << endl;
 	    if(logtimestamps) f << "logtimestamps" << endl;
 	    if(timestampstothesecond) f << "timestampstothesecond" << endl;
 	    if(logonline) f << "logonline" << endl;
@@ -622,6 +641,13 @@ void icqconf::save() {
 		    break;
 	    }
 
+	    switch(getsortmode()) {
+		case sort_by_status_and_activity: f << "sort_by_status_and_activity" << endl; break;
+		case sort_by_status_and_name: f << "sort_by_status_and_name" << endl; break;
+		case sort_by_activity: f << "sort_by_activity" << endl; break;
+		case sort_by_name: f << "sort_by_name" << endl; break;
+	    }
+            
 	    if(getmakelog()) f << "log" << endl;
 	    if(getproxyconnect()) f << "proxy_connect" << endl;
 	    if(getproxyssl()) f << "proxy_ssl" << endl;
@@ -1015,6 +1041,9 @@ void icqconf::setscreensocketpath(string path) {
 void icqconf::setquote(bool use) {
     quote = use;
 }
+void icqconf::setaskquit(bool faskquit) { 
+    askquit = faskquit; 
+} 
 
 void icqconf::setsockshost(const string &nsockshost) {
     int pos;
@@ -1278,6 +1307,7 @@ int icqconf::getstatuscolor(imstatus status) const {
         case dontdisturb : return getcolor(cp_clist_dnd);
         case occupied : return getcolor(cp_clist_occupied);
         case notavail : return getcolor(cp_clist_na);
+        case outforlunch : return getcolor(cp_clist_na);
         case away : return getcolor(cp_clist_away);
 	default : return getcolor(cp_main_text);
     }
@@ -1342,7 +1372,7 @@ void icqconf::commandline(int argc, char **argv) {
 	    debug = true;
 
 	} else if((args == "-v") || (args == "--version")) {
-	    cout << PACKAGE << " " << VERSION << endl
+	    cout << PACKAGE << " " << centerim::version << endl
 		<< "Written by Konstantin Klyagin, 2005. Currently maintained by the CenterIM community." << endl
 		<< "Built-in protocols are:";
 
@@ -1446,7 +1476,7 @@ const string &dest, const string &number) const {
 	    em.store(*ev);
 
 	    char buf[512];
-	    sprintf(buf, _("%s to %s has been put to the queue"),
+	    snprintf(buf, sizeof(buf), _("%s to %s has been put to the queue"),
 		streventname(ev->gettype()),
 		number.empty() ? c->getdesc().totext().c_str() : number.c_str());
 
@@ -1514,7 +1544,7 @@ void icqconf::externalstatuschange(char st, const string &proto) const {
 }
 
 void icqconf::usage() const {
-	cout << PACKAGE << " " << VERSION << endl;
+	cout << PACKAGE << " " << centerim::version << endl;
 	
     cout << endl << _("Usage: ") << argv0 << " [OPTION].." << endl;
 	
@@ -1568,6 +1598,11 @@ void icqconf::setgroupmode(icqconf::groupmode amode) {
     fgroupmode = amode;
 }
 
+void icqconf::setsortmode(icqconf::sortmode smode) {
+    fsortmode = smode;
+    icqcontacts::setsortmode(smode);
+}
+
 void icqconf::initmultiproto(bool p[], string buf, bool excludenochat) {
     string w;
     protocolname pname;
