@@ -67,6 +67,17 @@ const char *strrandomgroup(short unsigned int gr) {
     return "";
 }
 
+const char *strsortmode(icqconf::sortmode smode) {
+    switch(smode) {
+	case icqconf::sort_by_status_and_activity: return _("Status and Activity");
+	case icqconf::sort_by_status_and_name: return _("Status and Name");
+	case icqconf::sort_by_activity: return _("Activity");
+	case icqconf::sort_by_name: return _("Name");
+    }
+
+    return "";
+}
+
 bool icqface::sprofmanager(string &name, string &act) {
     dialogbox db;
     string buf, tname;
@@ -267,9 +278,14 @@ bool icqface::finddialog(imsearchparams &s, findsubject subj) {
 		case fschannel:
 		    services = gethook(s.pname).getservices(servicetype::groupchat);
 
+		    if((iservice = find(services.begin(), services.end(), s.service)) == services.end())
+			s.service = "";
+
 		    if(s.service.empty())
-		    if((iservice = services.begin()) != services.end())
+		    if((iservice = services.begin()) != services.end()) {
 			s.service = *iservice;
+			s.flexparams = gethook(s.pname).getsearchparameters(s.service);
+		    }
 		    break;
 	    }
 
@@ -507,6 +523,7 @@ bool icqface::finddialog(imsearchparams &s, findsubject subj) {
 			    if(++iservice == services.end())
 				iservice = services.begin();
 			    s.service = *iservice;
+			    s.flexparams = gethook(s.pname).getsearchparameters(s.service);
 			}
 			break;
 		    case 32: s.password = inputstr(_("Password: "), s.password, '*'); break;
@@ -522,7 +539,7 @@ bool icqface::finddialog(imsearchparams &s, findsubject subj) {
 		break;
 
 	    case 4:
-		if(s.pname == jabber && s.nick.find("@") == -1) {
+		if ((s.pname == jabber) && ((s.service.empty() && (s.nick.find("@") == -1)) || (s.nick.empty() && s.flexparams.empty()))) {
 		    status(_("Wrong Jabber ID!"));
 		} else {
 		    ret = finished = true;
@@ -1108,6 +1125,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     bool makelog = conf.getmakelog();
     bool askaway = conf.getaskaway();
     bool bidi = conf.getbidi();
+    bool askquit = conf.getaskquit();
     bool emacs = conf.getemacs();
     bool proxyconnect = conf.getproxyconnect();
     bool timestampstothesecond = conf.gettimestampstothesecond();
@@ -1140,6 +1158,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 
     icqconf::groupmode gmode = conf.getgroupmode();
     icqconf::colormode cm = conf.getcolormode();
+    icqconf::sortmode smode = conf.getsortmode();
 
     bool chatmode[protocolname_size], conv[protocolname_size],
 	entersends[protocolname_size], nonimonline[protocolname_size];
@@ -1194,6 +1213,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 	t.addleaff(i, 0, 20, _( " Enable bidirectional languages support : %s "), stryesno(bidi));
     #endif
 	t.addleaff(i, 0, 23, _(" Enable emacs bindings in text editor : %s "), stryesno(emacs));
+	t.addleaff(i, 0, 53, _(" Ask before quit : %s "), stryesno(askquit));
 
 	i = t.addnode(_(" Codepages conversion "));
 
@@ -1212,6 +1232,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 
 	i = t.addnode(_(" Contact list "));
 	t.addleaff(i, 0, 17, _(" Arrange contacts into groups : %s "), strgroupmode(gmode));
+	t.addleaff(i, 0, 54, _(" Sort contacts by : %s "), strsortmode(smode));
 	t.addleaff(i, 0,  6, _(" Hide offline users : %s "), stryesno(hideoffl));
 	t.addleaff(i, 0, 14, _(" Anti-spam: kill msgs from users not on the list : %s "), stryesno(antispam));
 	t.addleaff(i, 0, 51, _(" Anti-spam: ignore authorization requests: %s "), stryesno(dropauthreq));
@@ -1439,6 +1460,16 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 			if (dropauthreq && usingcaptcha)
 				dropauthreq = false;
 			break;
+		    case 53:
+		    	askquit = !askquit;
+			break;
+		    case 54:
+			smode =
+			    smode == icqconf::sort_by_status_and_activity ? icqconf::sort_by_status_and_name :
+			    smode == icqconf::sort_by_status_and_name ? icqconf::sort_by_activity :
+			    smode == icqconf::sort_by_activity ? icqconf::sort_by_name :
+				icqconf::sort_by_status_and_activity;
+			break;
   		}
   		break;
 	    case 1:
@@ -1470,6 +1501,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		}
 
 		conf.setbidi(bidi);
+		conf.setaskquit(askquit);
 		conf.setlogoptions(logtimestamps, logonline);
 
 		if(ptp) conf.setpeertopeer(ptpmin, ptpmax);
@@ -1480,6 +1512,9 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		    clist.rearrange();
 		}
 		conf.setcolormode(cm);
+		if (conf.getsortmode() != smode){
+		    conf.setsortmode(smode);
+		}
 
 		conf.setsmtphost(smtp);
 		conf.setbrowser(browser);
@@ -1956,6 +1991,8 @@ bool icqface::setljparams(imxmlevent *ev) {
 	svalues.push_back("usemask");
     }
 
+    ev->setfield("taglist", "");
+
     textwindow w(0, 0, sizeDlg.width, sizeDlg.height, conf.getcolor(cp_dialog_frame), TW_CENTERED);
     w.set_title(conf.getcolor(cp_dialog_highlight), _(" LiveJournal posting: attributes "));
     db.setwindow(&w, false);
@@ -1993,6 +2030,7 @@ bool icqface::setljparams(imxmlevent *ev) {
 
 	t.addleaff(i, 0, 21, _(" Music : %s "), ev->getfield("music").c_str());
 	t.addleaff(i, 0, 22, _(" Picture : %s "), ev->getfield("picture").empty() ? _("(default)") : ev->getfield("picture").c_str());
+	t.addleaff(i, 0, 25, _(" Tags : %s "), ev->getfield("taglist").c_str());
 
 	i = t.addnode(_(" Options "));
 	t.addleaff(i, 0, 30, _(" Disable auto-formatting : %s "), stryesno(ev->getfield("preformatted") == "1"));
@@ -2018,6 +2056,7 @@ bool icqface::setljparams(imxmlevent *ev) {
 		    case 21: LJP_STR("music", _("Currently playing: ")); break;
 		    case 22: LJP_LIST("picture", pictures, _("(default)")); break;
 		    case 23: LJP_STR("mood", _("Current mood: ")); break;
+		    case 25: LJP_STR("taglist", _("Tags for the entry: ")); break;
 		    case 30: LJP_BOOL("preformatted"); break;
 		    case 31: LJP_BOOL("noemail"); break;
 		    case 32: LJP_BOOL("nocomments"); break;
