@@ -36,6 +36,9 @@
 #include "irchook.h"
 #include "imexternal.h"
 #include "impgp.h"
+#ifdef HAVE_LIBOTR
+  #include "imotr.h"
+#endif  
 
 const char *stryesno(bool b) {
     return b ? _("yes") : _("no");
@@ -99,7 +102,7 @@ const char *strint(unsigned int i) {
     static char buf[64];
 
     if(i) {
-	sprintf(buf, "%lu", i);
+	snprintf(buf, sizeof(buf), "%lu", i);
     } else {
 	buf[0] = 0;
     }
@@ -237,7 +240,7 @@ void icqface::showtopbar() {
 
     attrset(conf.getcolor(cp_status));
     mvhline(0, 0, ' ', COLS);
-    mvprintw(0, 0, _(" CENTERIM %s  UNSENT: %lu"), VERSION, em.getunsentcount());
+    mvprintw(0, 0, _(" CENTERIM %s  UNSENT: %lu"), centerim::version, em.getunsentcount());
     mvprintw(0, COLS-buf.size()-1, "%s", buf.c_str());
 }
 
@@ -312,7 +315,7 @@ int icqface::contextmenu(icqcontact *c) {
 	actnames[ACT_PGPKEY] = c->getpgpkey().empty() ?
 	    _(" Assign PGP key..") : _(" Unassign PGP key");
 
-	sprintf(buf, _(" Turn PGP encryption %s"), c->getusepgpkey() ? "OFF" : "ON");
+	snprintf(buf, sizeof(buf), _(" Turn PGP encryption %s"), c->getusepgpkey() ? "OFF" : "ON");
 	actnames[ACT_PGPSWITCH] = buf;
 
 	if(c->getdesc().pname == rss)
@@ -427,6 +430,9 @@ int icqface::generalmenu() {
     m.additem(0, ACT_QUICKFIND, getmenuitem(_("Go to contact.."), 38, key_quickfind, section_contact));
     m.additem(0, ACT_DETAILS,   _(" Accounts.."));
     m.additem(0, ACT_CONF,      _(" CenterIM config options"));
+#ifdef HAVE_LIBOTR
+    m.additem(0, ACT_OTR,       _(" OTR options and fingerprints"));
+#endif
     m.additem(0, ACT_TRANSFERS, _(" File transfers monitor"));
     m.addline();
     m.additem(0, ACT_FIND,      _(" Find/add users"));
@@ -667,7 +673,7 @@ void icqface::fillcontactlist() {
 	    continue;
 	}
 
-	sc = SORTCHAR(c);
+	sc = icqcontacts::sort_order->sortstatus(*c);
 
 	groupchange =
 	    (conf.getgroupmode() != icqconf::nogroups) &&
@@ -680,7 +686,7 @@ void icqface::fillcontactlist() {
 	    ADDGROUP(0);
 
 	if(groupchange || (sc != '#')) {
-	    if(strchr("candifo", sc)) sc = 'O';
+	    if(strchr("candilfo", sc)) sc = 'O';
 
 	    if((sc != prevsc) || groupchange) {
 		switch(conf.getgroupmode()) {
@@ -1286,7 +1292,7 @@ void icqface::inforss(dialogbox &db, icqcontact *c) {
     char buf[512];
 
     if(mi.checkfreq) {
-	sprintf(buf, _("%lu minutes"), mi.checkfreq);
+	snprintf(buf, sizeof(buf), _("%lu minutes"), mi.checkfreq);
 	freq = buf;
 	if(mi.checklast) next = strdateandtime(mi.checklast+mi.checkfreq*60);
 	    else next = _("Now");
@@ -1544,6 +1550,7 @@ bool icqface::changestatus(vector<protocolname> &pnames, imstatus &st) {
 	mst.push_back(away);
 	mst.push_back(dontdisturb);
 	mst.push_back(notavail);
+	mst.push_back(outforlunch);
 	mst.push_back(occupied);
 	mst.push_back(freeforchat);
 	mst.push_back(invisible);
@@ -1821,7 +1828,7 @@ void icqface::modelist(contactstatus cs) {
 
 		    if(cs == csignore) {
 			char buf[512];
-			sprintf(buf, _("Remove the %d contacts from the contact list as well?"), muins.size());
+			snprintf(buf, sizeof(buf), _("Remove the %d contacts from the contact list as well?"), muins.size());
 			removecl = face.ask(buf, ASK_YES | ASK_NO, ASK_NO) == ASK_YES;
 		    }
 
@@ -1987,7 +1994,7 @@ void icqface::log(const char *fmt, ...) {
     char buf[1024];
 
     va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
+    vsnprintf(buf, sizeof(buf), fmt, ap);
     log((string) buf);
     va_end(ap);
 }
@@ -2068,7 +2075,7 @@ void icqface::status(const char *fmt, ...) {
     char buf[1024];
 
     va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
+    vsnprintf(buf, sizeof(buf), fmt, ap);
     status((string) buf);
     va_end(ap);
 }
@@ -2257,7 +2264,7 @@ void icqface::userinfoexternal(const imcontact &ic) {
 
 	if(i) {
 	    char cbuf[512];
-	    sprintf(cbuf, _("Result of the external action %s:"),
+	    snprintf(cbuf, sizeof(cbuf), _("Result of the external action %s:"),
 		(r.begin()+i-1)->second.c_str());
 
 	    string buf;
@@ -2689,6 +2696,10 @@ void icqface::peerinfo(int line, const imcontact &ic) {
 	text += string(4, ' ');
 #endif
 
+#ifdef HAVE_LIBOTR
+    text += "  OTR: " + otr.is_verified(ic.pname, c->getnick()) + " " + otr.get_msg_state(ic.pname, c->getnick());
+#endif
+
     text = (string) "[ " + text + " ]";
 
     kwriteatf(sizeWArea.x2-text.size()-1, sizeWArea.y1+line,
@@ -2965,7 +2976,7 @@ void icqface::fullscreenize(const imevent *ev) {
 	case imevent::outgoing: fmt = _("%s to %s, sent on %s"); break;
     }
 
-    sprintf(buf, fmt, streventname(ev->gettype()),
+    snprintf(buf, sizeof(buf), fmt, streventname(ev->gettype()),
 	ev->getcontact().totext().c_str(),
 	strdateandtime(ev->gettimestamp()).c_str());
 
@@ -3063,7 +3074,7 @@ bool icqface::histexec(imevent *&im) {
 
 	im = static_cast<imevent *> (mhist.getref(0));
 
-	sprintf(buf, _("History for %s, %d events total"),
+	snprintf(buf, sizeof(buf), _("History for %s, %d events total"),
 	    im->getcontact().totext().c_str(),
 	    mhist.getcount());
 
@@ -3281,6 +3292,9 @@ int icqface::contactskeys(verticalmenu &m, int k) {
 	    break;
 
 	case key_quit:
+	    if (conf.getaskquit())
+		if(face.ask("Really Quit?", ASK_YES | ASK_NO, ASK_NO) == ASK_NO)
+		    break;
 	    face.extk = ACT_QUIT;
 	    break;
 
@@ -3473,6 +3487,11 @@ int icqface::editmsgkeys(texteditor &e, int k) {
 	    face.editdone = false;
 	    face.next_chat(true);
 	    return -1;
+	case key_out_chat:
+	    face.editdone = false;
+	    face.next_chat(false);
+	    face.last_selected = (icqcontact* ) clist.at(0);
+	    return -1;
 	case key_info:
 	    cicq.userinfo(face.passinfo);
 	    break;
@@ -3492,6 +3511,20 @@ int icqface::editmsgkeys(texteditor &e, int k) {
 	    face.stay_in_chat = true;
 	    return -1;
 	    break;
+
+#ifdef HAVE_LIBOTR
+	case key_otr_start_session:
+	    otr.start_session(face.last_selected);
+	    face.stay_in_chat = true;
+	    return -1;
+	    break;
+	case key_otr_end_session:
+	    otr.end_session(face.last_selected);
+	    face.stay_in_chat = true;
+	    return -1;
+	    break;
+#endif
+
 	case key_quit:
 	    return -1;
     }
@@ -3531,7 +3564,7 @@ int icqface::findreskeys(dialogbox &db, int k) {
 }
 
 int icqface::statuskeys(verticalmenu &m, int k) {
-    char *status_order = "o_adncfi";    // Shortcuts for status
+    char *status_order = "o_adnlcfi";    // Shortcuts for status
     char *p = strchr(status_order, k);
     if (p)
 	return (1 + p - status_order);
@@ -3617,7 +3650,7 @@ void icqface::xtermtitle(const string &text) {
 
 	if(term == "xterm" || term == "Eterm" || term == "aterm"
 	|| term == "rxvt" || term.substr(0, 6) == "screen")
-	    cout << "\x1b]1;\x07\x1b]2;" << "centerim" << (text.empty() ? "" : (string) ": " + text) << "\x07" << flush;
+	    cout << "\x1b]1;\x07\x1b]0;" << "centerim" << (text.empty() ? "" : (string) ": " + text) << "\x07" << flush;
     }
 }
 
@@ -3627,7 +3660,7 @@ void icqface::xtermtitle(const char *fmt, ...) {
 	char buf[1024];
 
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	xtermtitle((string) buf);
 	va_end(ap);
     }
@@ -3655,7 +3688,7 @@ void icqface::icqprogress::log(const char *fmt, ...) {
     char buf[1024];
 
     va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
+    vsnprintf(buf, sizeof(buf), fmt, ap);
     va_end(ap);
 
     if(curline >= face.sizeDlg.height-1) {
